/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import Foundation
import UIKit
import VisioMoveEssential

class CameraMapViewController: UIViewController, VMEMapListener, VMELifeCycleListener, VMEAnimationCallback, VMECameraListener {
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var enableAnimationLabel: UILabel!
    @IBOutlet var enableAnimationSwitch: UISwitch!
    @IBOutlet var floorButton: UIButton!
    @IBOutlet var globalButton: UIButton!
    @IBOutlet var placeButton: UIButton!
    @IBOutlet var positionButton: UIButton!
    
    private var cameraUpdates: [VMECameraUpdate]? = nil
    private var cameraUpdateCount: Int = 0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        createCameraAnimations()
        // Do any additional setup after loading the view, typically from a nib.
        title = NSLocalizedString("CameraMapViewController.title", comment: "")
        enableAnimationLabel.text = NSLocalizedString("CameraMapViewController.animate", comment: "")

        floorButton.setTitle(NSLocalizedString("CameraMapViewController.floor", comment: ""), for: .normal)
        globalButton.setTitle(NSLocalizedString("CameraMapViewController.global", comment: ""), for: .normal)
        placeButton.setTitle(NSLocalizedString("CameraMapViewController.place", comment: ""), for: .normal)
        positionButton.setTitle(NSLocalizedString("CameraMapViewController.position", comment: ""), for: .normal)

        mapView?.mapListener = self
        mapView?.lifeCycleListener = self
        mapView?.cameraListener = self

        mapView?.loadMap()
        
        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    func createCameraAnimations() {
        cameraUpdates = [
            VMECameraUpdate(builderBlock: { builder in
                builder?.distanceRange = VMECameraDistanceRange(minRadius: 40, andMaxRadius: 200)
                builder?.pitch = VMECameraPitch(pitch: -10)
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["B2-UL00"]
                builder?.heading = VMECameraHeading(forPlaceID: "B2-UL00")
                builder?.pitch = VMECameraPitch(pitch: -30)
                builder?.distanceRange = VMECameraDistanceRange.cameraDistanceDefaultAltitude()
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.targets = [CAMERA_FOCAL_POINT]
                builder?.heading = VMECameraHeading(heading: 80)
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.heading = VMECameraHeading(heading: 45)
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["B1-UL00"]
                builder?.distanceRange = VMECameraDistanceRange(minRadius: 200, andMaxRadius: 250)
                builder?.heading = VMECameraHeading(forPlaceID: "B1-UL00")
                builder?.pitch = VMECameraPitch(pitch: -79)
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.distanceRange = VMECameraDistanceRange(minRadius: 20, andMaxRadius: 50)
            }),
            VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["outside"]
                builder?.heading = VMECameraHeading(forPlaceID: "outside")
                builder?.viewMode = VMEViewMode.global
                builder?.pitch = VMECameraPitch.cameraPitchDefault()
                builder?.distanceRange = VMECameraDistanceRange.cameraDistanceDefaultAltitude()
            })]
        
        cameraUpdateCount = 0
    }
    
    @IBAction func go(toPlace sender: Any) {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["B2-UL01-ID0010"]
                builder?.paddingTop = 10
                builder?.pitch = VMECameraPitch.cameraPitchDefault()
            })

        let lIsAnimated = enableAnimationSwitch.isOn
        (lIsAnimated) ? mapView?.animateCamera(lUpdate!) : mapView?.updateCamera(lUpdate!)
    }
    
    @IBAction func go(toFloor sender: Any) {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["B4-UL05"]
                builder?.heading = VMECameraHeading(forPlaceID: "B4")
                builder?.viewMode = VMEViewMode.floor
                builder?.pitch = VMECameraPitch(pitch: -79)
            })

        let lIsAnimated = enableAnimationSwitch.isOn
        (lIsAnimated) ? mapView?.animateCamera(lUpdate!) : mapView?.updateCamera(lUpdate!)
    }
    
    @IBAction func go(toGlobalMode sender: Any) {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["outside"]
                builder?.heading = VMECameraHeading(forPlaceID: "outside")
                builder?.viewMode = VMEViewMode.global
                builder?.pitch = VMECameraPitch(pitch: -30)
            })

        let lIsAnimated = enableAnimationSwitch.isOn
        (lIsAnimated) ? mapView?.animateCamera(lUpdate!) : mapView?.updateCamera(lUpdate!)
    }
    
    @IBAction func gotoPosition(_ sender: Any) {
        let lScene = VMESceneContext(buildingID: "B3", floorID: "B3-UL01")
        let lPos1 = VMEPosition(latitude: 45.740876918853147, longitude: 4.8805385544669795, altitude: 3, scene: lScene)

        let lPos2 = VMEPosition(latitude: 45.740797086277347, longitude: 4.8810472180340518, altitude: 3, scene: lScene)

        let lPos3 = VMEPosition(latitude: 45.740555146004695, longitude: 4.8806516436439189, altitude: 3, scene: lScene)

        let lPos4 = mapView?.getPlacePosition("B3-UL00-ID0070")
        
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
            builder?.targets = [lPos1 as Any, lPos2 as Any, lPos3 as Any, "B3-UL01-ID0021", lPos4 as Any]
                builder?.paddingTop = 50
                builder?.paddingBottom = 50
                builder?.paddingLeft = 50
                builder?.paddingRight = 50
                builder?.pitch = VMECameraPitch.cameraPitchCurrent()
            })

        let lIsAnimated = enableAnimationSwitch.isOn
        (lIsAnimated) ? mapView?.animateCamera(lUpdate!) : mapView?.updateCamera(lUpdate!)
    }
    
    func cameraStartPosition() {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["outside"]
            builder?.heading = VMECameraHeading(forPlaceID: "outside")
            builder?.viewMode = VMEViewMode.global
                builder?.pitch = VMECameraPitch(pitch: -30)
        })!
        mapView?.updateCamera(lUpdate)
    }
    
    func cameraFirstAnimation() {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
                builder?.targets = ["outside"]
                builder?.pitch = VMECameraPitch(pitch: -79)
        })!
        mapView?.animateCamera(lUpdate, duration: 0.7, callback: self)
    }
    
    // MARK: - VMELifeCycleListener
    func mapDidInitializeEngine(_ mapView: VMEMapView?) {
        print("Map did initialize the rendering engine")
    }

    func mapDidGainFocus(_ mapView: VMEMapView?) {
        print("Map displayed")
    }

    func mapDidLoad(_ mapView: VMEMapView?) {
        print("Map did load")
        cameraStartPosition()
        cameraFirstAnimation()
    }
    
    func map(_ mapView: VMEMapView?, didDisplayRoute result: VMERouteResult?) {
        var lMsg: String? = nil
        if let length = result?.length {
            lMsg = "Route displayed: \(length)"
        }
        print("\(lMsg ?? "")")
    }

    func map(_ mapView: VMEMapView?, didDisplayPlaceInfo placeID: String?) {
        let lMsg = "Place info displayed: \(placeID ?? "")"
        print("\(lMsg)")
    }

    func mapDidDisplaySearch(_ mapView: VMEMapView?) {
        print("Search displayed")
    }

    func mapDidDisplayRouteSetup(_ mapView: VMEMapView?) {
        print("Route setup displayed")
    }
    
    // MARK: - VMEMapListener
    func map(_ mapView: VMEMapView?, sceneDidUpdate scene: VMESceneContext?, with viewMode: VMEViewMode) {
        var lMsg: String? = nil
        if let description = scene?.description {
            lMsg = String(format: "Mode: %ld Scene: %@", viewMode.rawValue, description)
        }
        print("\(lMsg ?? "")")
    }

    func map(_ mapView: VMEMapView?, didReceiveTapGesture position: VMEPosition?) {
        if position == nil {
            let lMsg = "mapTapGesture outside of map: "
            print("\(lMsg)")
        } else {
            var lMsg: String? = nil
            if let position = position {
                lMsg = "mapTapGesture: Position: \(position)"
            }
            print("\(lMsg ?? "")")
        }
    }
    
    // MARK: - VMEAnimationListener
    func didFinish() {
        cameraUpdateCount += 1
        if cameraUpdateCount < cameraUpdates!.count {
            perform(#selector(scheduleCameraAnimation), with: nil, afterDelay: 0.25)
        }
    }

    @objc func scheduleCameraAnimation() {
        let cameraUpdate = cameraUpdates![cameraUpdateCount]
        mapView?.animateCamera(cameraUpdate, duration: 0.7, callback: self)
    }
    
    // MARK: - VMECameraListener
    func mapCameraDidMove(_ mapView: VMEMapView? ) {
        let lCameraContext : VMECameraContext = (mapView?.getCameraContext())!
        print(lCameraContext)
        var reasonStr : String = ""
        let cameraMoveReason : VMECameraMoveReason = (mapView?.getCameraMoveReason())!
        switch(cameraMoveReason) {
            case VMECameraMoveReason.animation:
                reasonStr = "Animation"
                break;
            case VMECameraMoveReason.developerAnimation:
                reasonStr = "Developer animation"
                break;
            case VMECameraMoveReason.gesture:
                reasonStr = "Gesture"
                break;
            default:
                reasonStr = "Unknown"
        }
        print(reasonStr)
    }
}
