/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import UIKit
import VisioMoveEssential

class LocationMapViewController: UIViewController, CLLocationManagerDelegate {
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var enableLocationLabel: UILabel!
    @IBOutlet var simulateLocationButton: UIButton!

    private var locationManager: CLLocationManager!
    private var fakeUserLocations: [CLLocation]?
    private var currentFakeUserLocationsIndex = 0
    private var locationUpdateDelayInSecs = 0
    
    deinit {
        if (locationManager != nil) {
            locationManager?.delegate = nil
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        title = NSLocalizedString("LocationMapViewController.title", comment: "")
        enableLocationLabel.text = NSLocalizedString("LocationMapViewController.location", comment: "")
        simulateLocationButton.setTitle(NSLocalizedString("LocationMapViewController.simulateLocation", comment: ""), for: .normal)
        initFakeLocations()
        locationUpdateDelayInSecs = 2

        mapView?.loadMap()

        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    func initFakeLocations() {
        fakeUserLocations = [CLLocation]()
        fakeUserLocations?.append(getLocationLat(45.7414425, lon: 4.8817110, alt: 0, bearing: 0, accuracy: 25)!)
        fakeUserLocations?.append(getLocationLat(45.7415096, lon: 4.8813725, alt: 0, bearing: 0, accuracy: 20)!)
        fakeUserLocations?.append(getLocationLat(45.741124, lon: 4.8812375, alt: 0, bearing: 0, accuracy: 15)!)
        fakeUserLocations?.append(getLocationLat(45.7408424, lon: 4.8811657, alt: 0, bearing: 0, accuracy: 10)!)
        fakeUserLocations?.append(getLocationLat(45.7409480, lon: 4.8805126, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7406672, lon: 4.8804190, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405301, lon: 4.8804921, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405301, lon: 4.8804921, alt: 4, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405229, lon: 4.8806677, alt: 4, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7407234, lon: 4.8810041, alt: 4, bearing: 0, accuracy: 5)!)
    }
    
    func getLocationLat(_ lat: Double, lon: Double, alt: Double, bearing: Double, accuracy accuarcy: Double) -> CLLocation? {
        let lCLLocation = CLLocation(
            coordinate: CLLocationCoordinate2DMake(CLLocationDegrees(lat), CLLocationDegrees(lon)),
            altitude: CLLocationDistance(alt /* will determine which floor is associated with the location */),
            horizontalAccuracy: CLLocationAccuracy(accuarcy /* will control the size of the accuracy disc */),
            verticalAccuracy: CLLocationAccuracy(0 /* when negative, will force the location to be outside */),
            course: CLLocationDirection(bearing /* will update the camera heading */),
            speed: CLLocationSpeed(5), timestamp: Date())
        return lCLLocation
    }
    
    @IBAction func startLocationSimulation(_ sender: Any) {
        currentFakeUserLocationsIndex = 0
        Timer.scheduledTimer(timeInterval: 0, target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
    }
    
    @objc func simulateLocation(_ pTimer: Timer?) {
        if currentFakeUserLocationsIndex < fakeUserLocations!.count {
            let lLoc : CLLocation = (fakeUserLocations?[currentFakeUserLocationsIndex])!
            let lLocations : [CLLocation] = [lLoc]
            if (locationManager == nil) {
                locationManager = CLLocationManager()
            }
            locationManager(locationManager!, didUpdateLocations: lLocations)
            currentFakeUserLocationsIndex += 1
            Timer.scheduledTimer(timeInterval: TimeInterval(locationUpdateDelayInSecs), target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
        }
    }
    
    @IBAction func enableLocation(_ sender: Any) {

        if !(locationManager != nil) {
            locationManager = CLLocationManager()
            locationManager.delegate = self
            locationManager.desiredAccuracy = kCLLocationAccuracyBestForNavigation
            // This is required in iOS8 in order to prompt the user to accept
            // to use the localisation
            if locationManager.responds(to: #selector(CLLocationManager.requestWhenInUseAuthorization)) {
                locationManager.requestWhenInUseAuthorization()
            }
        }

        if (sender as AnyObject).isOn {
            locationManager.startUpdatingLocation()
        } else {
            locationManager.stopUpdatingLocation()
            mapView?.update(nil)
        }
    }
    
    // MARK: - CLLocationManagerDelegate
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        let lCLLocation = locations.last!
        let lVMELocation : VMELocation? = (mapView?.createLocation(from: lCLLocation))
        if(lVMELocation != nil){
            mapView?.update(lVMELocation) // lVMELocation can be nil
        }        
    }
}

