/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import UIKit
import VisioMoveEssential

class VMEArrowView: UIView {
    var title: UILabel?
    var categories: UILabel?
    var close: UIButton?
    
    let kArrowHeight = 15
    override init(frame: CGRect) {
        super.init(frame: frame)
        title = UILabel(frame: CGRect(x: 0, y: 0, width: frame.size.width, height: 20))
        title?.textAlignment = .center
        addSubview(title!)
        categories = UILabel(frame: CGRect(x: 0, y: 20, width: frame.size.width, height: 20))
        categories?.textAlignment = .left
        categories?.numberOfLines = 2
        addSubview(categories!)
        close = UIButton(type: .custom)
        close?.frame = CGRect(x: 0, y: 60, width: frame.size.width, height: 20)
        addSubview(close!)
        backgroundColor = UIColor.clear
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func draw(_ rect: CGRect) {
        let context = UIGraphicsGetCurrentContext()
        let fillPath = UIBezierPath()
        fillPath.move(to: CGPoint(x: 0, y: 0))
        fillPath.addLine(to: CGPoint(x: bounds.size.width, y: 0))
        fillPath.addLine(to: CGPoint(x: bounds.size.width, y: bounds.size.height - CGFloat(kArrowHeight)))
        fillPath.addLine(to: CGPoint(x: Int(bounds.size.width) / 2 + (kArrowHeight / 2), y: Int(bounds.size.height) - kArrowHeight))
        fillPath.addLine(to: CGPoint(x: bounds.size.width / 2, y: bounds.size.height))
        fillPath.addLine(to: CGPoint(x: Int(bounds.size.width) / 2 - (kArrowHeight / 2), y: Int(bounds.size.height) - kArrowHeight))
        fillPath.addLine(to: CGPoint(x: 0, y: Int(bounds.size.height) - kArrowHeight))
        fillPath.close()

        context?.addPath(fillPath.cgPath)
        context?.setFillColor(UIColor.lightGray.cgColor)
        context?.fillPath()

        let shape = CAShapeLayer()
        shape.path = fillPath.cgPath
        shape.lineWidth = 2
        shape.strokeColor = UIColor.gray.cgColor
        shape.fillColor = UIColor.clear.cgColor
        shape.frame = bounds
        layer.addSublayer(shape)
    }
}

private var sLonePineOverlayViewID = "lonePineOverlayView"
private var sPlaceOverlayViewID = "placeOverlayView"

class OverlayViewController: UIViewController, VMEPlaceListener, VMEBuildingListener {
    @IBOutlet var infoViewLabel: UILabel!
    @IBOutlet var infoViewAddedSwitch: UISwitch!
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var moveOverlayButton: UIButton!
    
    private var mLonePineView: VMEArrowView?
    private var mPlaceOverlayView: VMEArrowView?
    private var mOverlays: NSMutableDictionary = [:]
    private var mIsPlaceMarkerDisplayed = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        title = NSLocalizedString("OverlayViewController.title", comment: "")
        infoViewLabel.text = NSLocalizedString("OverlayViewController.toggleOverlays", comment: "")
        moveOverlayButton.setTitle(NSLocalizedString("OverlayViewController.moveOverlays", comment: ""), for: .normal)
        moveOverlayButton.isEnabled = false

        // Create the place overlay view.
        do {
            mPlaceOverlayView = VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100))
            mPlaceOverlayView?.close?.addTarget(self, action: #selector(closeMarkerPlace(_:)), for: .touchUpInside)
            mPlaceOverlayView?.close?.setTitle("Close", for: .normal)
        }

        // Must be called in order for the map to be loaded
        mapView?.loadMap()
        
        mOverlays = NSMutableDictionary()
        mOverlays.setObject(VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100)), forKey: "Overlay1" as NSCopying)
        mOverlays.setObject(VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100)), forKey: "Overlay2" as NSCopying)
        mOverlays.setObject(VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100)), forKey: "Overlay3" as NSCopying)
        mOverlays.setObject(VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100)), forKey: "Overlay4" as NSCopying)
        mOverlays.setObject(VMEArrowView(frame: CGRect(x: 0, y: 0, width: 200, height: 100)), forKey: "Overlay5" as NSCopying)

        mIsPlaceMarkerDisplayed = false

        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    @IBAction func infoViewSwitch(_ sender: Any) {
        let lSwitch = sender as? UISwitch
        if lSwitch?.isOn == true {
            var lView: VMEArrowView?

            // overlay on outside place
            lView = mOverlays["Overlay1"] as? VMEArrowView
            lView?.title?.text = "Lone Pine Overlay View"
            lView?.close?.isHidden = true
            mapView?.addOverlayViewID("Overlay1", view: lView!, placeID: "outside-lonepine")

            // overlay on indoor place
            lView = mOverlays["Overlay2"] as? VMEArrowView
            lView?.title?.text = "B4-UL00-ID0015"
            lView?.close?.isHidden = true
            mapView?.addOverlayViewID("Overlay2", view: lView!, placeID: "B4-UL00-ID0015")

            // overlay on underground place
            lView = mOverlays["Overlay3"] as? VMEArrowView
            lView?.title?.text = "B2-LL01-ID0008"
            lView?.close?.isHidden = true
            mapView?.addOverlayViewID("Overlay3", view: lView!, placeID: "B2-LL01-ID0008")
            
            // One with lat/lon pos in the first floor of the offices
            lView = mOverlays["Overlay4"] as? VMEArrowView
            lView?.title?.text = "45.74271 N 4.88076 E"
            lView?.close?.isHidden = true
            let lPos = VMEPosition(latitude: 45.74271, longitude: 4.88076, altitude: 0, scene: VMESceneContext(buildingID: "B4", floorID: "B4-UL01"))!
            mapView?.addOverlayViewID("Overlay4", view: lView!, position: lPos)

            // One with lat/lon pos outside
            lView = mOverlays["Overlay5"] as? VMEArrowView
            lView?.title?.text = "45.74156 N 4.88192 E"
            lView?.close?.isHidden = true
            let lPosOutside = VMEPosition(latitude: 45.74156, longitude: 4.88192, altitude: 0, scene: VMESceneContext.init())!
            mapView?.addOverlayViewID("Overlay5", view: lView!, position: lPosOutside)
            
            moveOverlayButton.isEnabled = true
            
        } else {
            
            for (lKey, _) in mOverlays as! [String:VMEArrowView] {
                mapView?.removeOverlayViewID(lKey)
            }
            moveOverlayButton.isEnabled = false
        }
    }
    
    @IBAction func moveOverlay(_ sender: Any) {
        mapView?.setOverlayViewID("Overlay5", placeID: "B1")

        let lPosOutside = VMEPosition(latitude: 45.74156, longitude: 4.88192, altitude: 0, scene: VMESceneContext.init())!

        mapView?.setOverlayViewID("Overlay1", position: lPosOutside)
    }
    
    @IBAction func closeMarkerPlace(_ sender: Any) {
        mapView?.removeOverlayViewID(sPlaceOverlayViewID)
        mIsPlaceMarkerDisplayed = false
    }
    
    func map(_ mapView: VMEMapView?, didSelectPlace placeID: String?, with position: VMEPosition?) -> Bool {
        let lPlace = mapView?.getPlace(placeID!)
        let lTitle = (lPlace) != nil ? lPlace?.name : placeID

        var lCategories = ""
        if let lPlaceCategories = lPlace?.categories {
            for lCategoryID : String in lPlaceCategories {
                let lCategory = mapView?.getCategory(lCategoryID)
                if lCategories.count > 0 {
                    lCategories = "\(lCategories),"
                }
                if let name = lCategory?.name {
                    lCategories = "\(lCategories) \(name)"
                }
            }
        }

        mPlaceOverlayView?.title?.text = lTitle
        mPlaceOverlayView?.categories?.text = lCategories
            
        if mIsPlaceMarkerDisplayed {
            self.mapView?.setOverlayViewID(sPlaceOverlayViewID, position: position!)
        } else {
            self.mapView?.addOverlayViewID(sPlaceOverlayViewID, view: mPlaceOverlayView!, position: position!, anchorMode: VMEAnchorMode.bottomCenter)
            mIsPlaceMarkerDisplayed = true
        }

        let lUpdate = VMECameraUpdate(builderBlock: { builder in
            builder?.targets = [position as Any]
                builder?.paddingTop = 100
            })
        self.mapView?.animateCamera(lUpdate!)

        return true
    }
    
    // MARK: - VMEBuildingListener
    func map(_ mapView: VMEMapView?, didSelectBuilding buildingID: String?, with position: VMEPosition?) -> Bool {
        let lUpdate = VMECameraUpdate(builderBlock: { builder in
            builder?.targets = [buildingID as Any]
        })!
        self.mapView?.updateCamera(lUpdate)

        return true
    }
    
}

