/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import UIKit
import VisioMoveEssential

class RoutingCustomLocationTrackerController: UIViewController, CLLocationManagerDelegate, VMEComputeRouteCallback, VMELocationTrackingModeListener {
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var mCustomTrackerButton: UISwitch!
    @IBOutlet var mDisplayCompassHeadingButton: UISwitch!
    @IBOutlet var simulateLocationButton: UIButton!
    @IBOutlet var customTrackerLabel: UILabel!
    @IBOutlet var displayCompassHeadingLabel: UILabel!

    private var locationManager: CLLocationManager!
    private var fakeUserLocations: [CLLocation]?
    private var fakeRouteLocations: [CLLocation]?
    private var currentFakeUserLocationsIndex = 0
    private var locationUpdateDelayInSecs = 0
    private var mCurrentTrackingMode: VMELocationTrackingMode?
    private var mCustomTracker: CustomLocationTracker?
    private var mLocation: VMELocation?
    
     deinit {
        if (locationManager != nil) {
            locationManager?.delegate = nil
        }
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        title = NSLocalizedString("RoutingCustomLocationTracker.title", comment: "")
        customTrackerLabel.text = NSLocalizedString("RoutingCustomLocationTracker.customLocationTracker", comment: "")
        simulateLocationButton.setTitle(NSLocalizedString("RoutingCustomLocationTracker.simulateLocation", comment: ""), for: .normal)
        
        displayCompassHeadingLabel.text = NSLocalizedString("RoutingCustomLocationTracker.displayCompassHeading", comment: "")

        mCurrentTrackingMode = VMELocationTrackingMode.none

        mCustomTracker = CustomLocationTracker()

        initFakeLocations()
        initRouteLocations()

        mapView?.locationTrackingModeListener = self
        locationUpdateDelayInSecs = 2

        mapView?.loadMap()
        
        let lToggleTrackingModes : [VMELocationTrackingModeObject]? = [
            VMELocationTrackingModeObject(withEnum: VMELocationTrackingMode.none),
            VMELocationTrackingModeObject(withEnum: VMELocationTrackingMode.follow),
            VMELocationTrackingModeObject(withEnum: VMELocationTrackingMode.custom)
        ]
        mapView?.setLocationTrackingButtonToggleModes(lToggleTrackingModes!)
        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    @IBAction func toggleCustomTCK(_ sender: Any) {
        let lNewTrackingMode = mCustomTrackerButton.isOn ? VMELocationTrackingMode.custom : VMELocationTrackingMode.none
        mapView?.setLocationTrackingMode(lNewTrackingMode)
    }
    
    @IBAction func toggleDisplayCompassHeading(_ sender: Any) {
        mapView?.setCompassHeadingMarkerVisible(mDisplayCompassHeadingButton.isOn)
    }

    func computeRouteFromLocation() {
        // The altitude determines the layer the position is associated with
        let lPositionStart = mapView?.createPosition(from: (fakeRouteLocations?[0])!)
        let lPositionWaypoint = mapView?.createPosition(from: (fakeRouteLocations?[1])!)
        let lPositionWaypoint2 = mapView?.createPosition(from: (fakeRouteLocations?[2])!)
        let lPositionWaypoint3 = mapView?.createPosition(from: (fakeRouteLocations?[3])!)

        let lDests = [lPositionWaypoint, lPositionWaypoint2, lPositionWaypoint3].compactMap { $0 }
        let lDestOrder = VMERouteDestinationsOrder.inOrder
        let lRouteRequest = VMERouteRequest(requestType: VMERouteRequestType.fastest, destinationsOrder: lDestOrder)
        lRouteRequest?.setOrigin(lPositionStart)
        lRouteRequest?.addDestinations(lDests)

        mapView?.computeRoute(lRouteRequest!, callback: self)
    }
    
    func initFakeLocations() {
        fakeUserLocations = [CLLocation]();
        //SEGMENT 1
        fakeUserLocations?.append(getLocationLat(45.7414425, lon: 4.8817110, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74146018470637, lon: 4.8816267137889, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.741503242232234, lon: 4.881454566005357, alt: 0, bearing: 0, accuracy: 5)!)

        //SEGMENT 2
        fakeUserLocations?.append(getLocationLat(45.74146648908168, lon: 4.881362728302338, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7413581006088, lon: 4.881325249556848, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.741250622518066, lon: 4.881290955936187, alt: 0, bearing: 0, accuracy: 5)!)
    
        //LOST SEGMENT
        fakeUserLocations?.append(getLocationLat(45.74159580963391, lon: 4.880611763935984, alt: 0, bearing: 0, accuracy: 5)!)
        
        //SEGMENT 3
        fakeUserLocations?.append(getLocationLat(45.74122681279279, lon: 4.881281834037413, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.741161167362506, lon: 4.881259036449916, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74111573850439, lon: 4.881241967302465, alt: 0, bearing: 0, accuracy: 5)!)

        //SEGMENT 4
        fakeUserLocations?.append(getLocationLat(45.74111185234072, lon: 4.88126743386255, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.741082109492496, lon: 4.881358671707478, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.740943741367786, lon: 4.881399690958989, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7408876462151, lon: 4.881378630427969, alt: 0, bearing: 0, accuracy: 5)!)
        
        //SEGMENT 5
        fakeUserLocations?.append(getLocationLat(45.740887806100226, lon: 4.881352023515476, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74087755098924, lon: 4.881280439421292, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.740872365892706, lon: 4.881280439421292, alt: 0, bearing: 0, accuracy: 5)!)

        //LOST SEGMENT 2
        fakeUserLocations?.append(getLocationLat(45.740732552168815, lon: 4.880405907263137, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74067635261162, lon: 4.880451422529728, alt: 0, bearing: 0, accuracy: 5)!)
        
        //SEGMENT 6
        fakeUserLocations?.append(getLocationLat(45.74086110693569, lon: 4.881218688870884, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74083808686493, lon: 4.881192991978083, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.740824658458685, lon: 4.881192991978083, alt: 0, bearing: 0, accuracy: 5)!)
        
        //SEGMENT 7
        fakeUserLocations?.append(getLocationLat(45.74083085871918, lon: 4.881128139191, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.740869812520074, lon: 4.881100961623086, alt: 0, bearing: 0, accuracy: 5)!)

        //SEGMENT 8
        fakeUserLocations?.append(getLocationLat(45.74088103914783, lon: 4.88106341625494, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.740890381049525, lon: 4.881013240897742, alt: 0, bearing: 0, accuracy: 5)!)

        fakeUserLocations?.append(getLocationLat(45.74091548961581, lon: 4.880795148700796, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.74094078061023, lon: 4.880569249628036, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7409480, lon: 4.8805126, alt: 0, bearing: 0, accuracy: 5)!)

    }
    
    func initRouteLocations() {
        fakeRouteLocations = [CLLocation]()
        fakeRouteLocations?.append(getLocationLat(45.7414425, lon: 4.8817110, alt: 0, bearing: 0, accuracy: 5)!)
        fakeRouteLocations?.append(getLocationLat(45.7415096, lon: 4.8813725, alt: 0, bearing: 0, accuracy: 5)!)
        fakeRouteLocations?.append(getLocationLat(45.7408424, lon: 4.8811657, alt: 0, bearing: 0, accuracy: 5)!)
        fakeRouteLocations?.append(getLocationLat(45.7409480, lon: 4.8805126, alt: 0, bearing: 0, accuracy: 5)!)
    }

    func getLocationLat(_ lat: Double, lon: Double, alt: Double, bearing: Double, accuracy accuarcy: Double) -> CLLocation? {
        let lCLLocation = CLLocation(coordinate: CLLocationCoordinate2DMake(CLLocationDegrees(lat), CLLocationDegrees(lon)), altitude: CLLocationDistance(alt /* will determine which floor is associated with the location */), horizontalAccuracy: CLLocationAccuracy(accuarcy /* will control the size of the accuracy disc */), verticalAccuracy: CLLocationAccuracy(0 /* when negative, will force the location to be outside */), course: CLLocationDirection(bearing /* will update the camera heading */), speed: CLLocationSpeed(5), timestamp: Date())
        return lCLLocation
    }

    
    @IBAction func startLocationSimulation(_ sender: Any) {
        currentFakeUserLocationsIndex = 0
        computeRouteFromLocation()
        Timer.scheduledTimer(timeInterval: 0, target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
    }
    
    @objc func simulateLocation(_ pTimer: Timer?) {
        if currentFakeUserLocationsIndex < fakeUserLocations!.count {
            let lLoc = fakeUserLocations?[currentFakeUserLocationsIndex]
            let lLocations = [lLoc].compactMap { $0 }
            if (locationManager == nil) {
                locationManager = CLLocationManager()
            }
            locationManager(locationManager!, didUpdateLocations: lLocations)
            currentFakeUserLocationsIndex += 1
            Timer.scheduledTimer(timeInterval: TimeInterval(locationUpdateDelayInSecs), target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
        }
    }
    
    // MARK: - CLLocationManagerDelegate
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        let lCLLocation = locations.last
        mLocation = mapView?.createLocation(from: lCLLocation!)
        if mCurrentTrackingMode == VMELocationTrackingMode.custom {
            mCustomTracker?.trackLocation(withMap: mapView, location: mLocation)
        }
        mapView?.update(mLocation)
    }
    
    // MARK: - VMEComputeRouteCallback
    func computeRouteDidFinish(_ mapView: VMEMapView?, parameters routeRequest: VMERouteRequest?, result routeResult: VMERouteResult?) -> Bool {
        if let length = routeResult?.length {
            print(String(format: "computeRouteDidFinish duration: %.0fmins and length: %.0fm ", (routeResult?.duration ?? 0) / 60, length))
        }
        return true
    }
    
    func computeRouteDidFail(_ mapView: VMEMapView?, parameters routeRequest: VMERouteRequest?, error: String?) {
        print("computeRouteDidFail error: \(error ?? "")")
    }
    
    // MARK: - VMELocationTrackingModeListener
    func map(_ mapView: VMEMapView?, didUpdate locationTrackingMode: VMELocationTrackingMode) {
        mCurrentTrackingMode = locationTrackingMode
        if locationTrackingMode != VMELocationTrackingMode.custom {
            if mLocation != nil {
                let lUpdate = VMECameraUpdate(builderBlock: { builder in
                        builder?.targets = [CAMERA_FOCAL_POINT]
                        builder?.pitch = VMECameraPitch(pitch: -79)
                })!
                self.mapView?.animateCamera(lUpdate, duration: 0.7, callback: nil)
            }
            mCustomTrackerButton.isOn = false
        } else {
            mCustomTrackerButton.isOn = true
        }
    }
    
}

