/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import UIKit
import VisioMoveEssential

/// Demonstrates how to compute a route

class RoutingMapViewController: UIViewController, VMELifeCycleListener, VMEComputeRouteCallback {
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var enableAccessibleRouteLabel: UILabel!
    @IBOutlet var enableAccessibleRouteSwitch: UISwitch!
    @IBOutlet var computeRoute: UIButton!
    @IBOutlet var computeRouteFromLocation: UIButton!
    @IBOutlet var enableOptimizeSwitch: UISwitch!
    @IBOutlet var enableOptmizeLabel: UILabel!
    @IBOutlet weak var discardNavigation: UIButton!
    @IBOutlet var showNavigationHeaderSwitch: UISwitch!
    @IBOutlet var showNavigationHeaderLabel: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.

        title = NSLocalizedString("RoutingMapViewController.title", comment: "")
        enableAccessibleRouteLabel.text = NSLocalizedString("RoutingMapViewController.accessible", comment: "")
        enableOptmizeLabel.text = NSLocalizedString("RoutingMapViewController.optimize", comment: "")
        showNavigationHeaderLabel.text = NSLocalizedString("RoutingMapViewController.showNavigationHeader", comment: "")
        computeRoute.setTitle(NSLocalizedString("RoutingMapViewController.createRouteFromPlace", comment: ""), for: .normal)
        computeRouteFromLocation.setTitle(NSLocalizedString("RoutingMapViewController.createRouteFromLocation", comment: ""), for: .normal)
        discardNavigation.setTitle(NSLocalizedString("RoutingMapViewController.discardNavigation", comment: ""), for: .normal)
        
        mapView?.lifeCycleListener = self

        showNavigationHeaderSwitch.isOn = ((mapView?.getNavigationHeaderViewVisible()) != nil)
        mapView?.loadMap()

        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    @IBAction func showNavigationHeader(_ sender: Any) {
        mapView?.setNavigationHeaderViewVisible(showNavigationHeaderSwitch.isOn)
    }
    
    @IBAction func computeRoute(fromPlace sender: Any?) {
        let lIsAccessible = enableAccessibleRouteSwitch.isOn
        let lDestOrder = enableOptimizeSwitch.isOn ? VMERouteDestinationsOrder.optimal : VMERouteDestinationsOrder.inOrder
        let lRouteRequest = VMERouteRequest(requestType: VMERouteRequestType.fastest, destinationsOrder: lDestOrder, accessible: lIsAccessible)
        lRouteRequest?.setOrigin("B1-UL00-ID0039")
        lRouteRequest?.addDestinations(["B4-UL05-ID0032", "B2-LL01-ID0011", "B3-UL00-ID0070"])

        mapView?.computeRoute(lRouteRequest!, callback: self)
    }
    
    @IBAction func computeRoute(fromLocation sender: Any) {

        let lIsAccessible = enableAccessibleRouteSwitch.isOn
        let lDestOrder = enableOptimizeSwitch.isOn ? VMERouteDestinationsOrder.optimal : VMERouteDestinationsOrder.inOrder
        let lRouteRequest = VMERouteRequest(requestType: VMERouteRequestType.fastest, destinationsOrder: lDestOrder, accessible: lIsAccessible)

        // The altitude determines the layer the position is associated with
        var lCLLocation = CLLocation(coordinate: CLLocationCoordinate2DMake(CLLocationDegrees(45.7431), CLLocationDegrees(4.8832)), altitude: CLLocationDistance(0), horizontalAccuracy: CLLocationAccuracy(5), verticalAccuracy: CLLocationAccuracy(0), course: CLLocationDirection(-1), speed: CLLocationSpeed(5), timestamp: Date())
        let lPositionStart = mapView?.createPosition(from: lCLLocation)
        
        lRouteRequest?.setOrigin(lPositionStart)

        lCLLocation = CLLocation(coordinate: CLLocationCoordinate2DMake(CLLocationDegrees(45.7409978), CLLocationDegrees(4.8806556)), altitude: CLLocationDistance(0), horizontalAccuracy: CLLocationAccuracy(5), verticalAccuracy: CLLocationAccuracy(0), course: CLLocationDirection(-1), speed: CLLocationSpeed(5), timestamp: Date())
        let lPositionWaypoint = mapView?.createPosition(from: lCLLocation)

        lRouteRequest?.addDestinations([
            lPositionWaypoint as Any,
            "B4-UL05-ID0032",
            "B2-LL01-ID0011",
            "B3-UL00-ID0070"
        ])

        mapView?.computeRoute(lRouteRequest!, callback: self)

    }
    
    @IBAction func discardNavigationAction(_ sender: Any) {
        mapView?.setFocusOnMap()
    }
    
    // MARK: - VMELifeCycleListener
    func mapDidLoad(_ mapView: VMEMapView?) {
        computeRoute(fromPlace: nil)
    }

    // MARK: - VMEComputeRouteCallback
    func computeRouteDidFinish(_ mapView: VMEMapView?, parameters routeRequest: VMERouteRequest?, result routeResult: VMERouteResult?) -> Bool {
        if let length = routeResult?.length {
            print(String(format: "computeRouteDidFinish duration: %.0fmins and length: %.0fm ", (routeResult?.duration ?? 0) / 60, length))
        }
        return true
    }
    
    func computeRouteDidFail(_ mapView: VMEMapView?, parameters routeRequest: VMERouteRequest?, error: String?) {
        print("computeRouteDidFail error: \(error ?? "")")
    }

}
