/*
* Copyright (c) Visioglobe SAS. All rights reserved.
*
* This file is subject to the terms and conditions defined in
* file 'LICENSE.txt', which is part of this source code package.
*/

import UIKit
import VisioMoveEssential

private var kLonMoverArr = [0.0002, 0.0, -0.0002, 0.0002, 0.0, -0.0002]
private var kLatMoverArr = [0.0002, 0.0004, 0.0002, -0.0002, -0.0004, -0.0002]
private var catMoverIndex = 0

/// This location demo shows how a location provider can be integrated.
///
/// To simulated location positions that fall within the Visio Island data set, we
/// have included "visio_island_simulate_location.gpx".
///
/// See here for how to use a gpx within xcode: http://apple.co/1TrE497
///
class StatisticsViewController: UIViewController, CLLocationManagerDelegate {
    @IBOutlet var mapView: VMEMapView!
    @IBOutlet var enableLocationLabel: UILabel!
    @IBOutlet var simulateLocationButton: UIButton!
    @IBOutlet var placeAddLabel: UILabel!
    var mCatMoverTimer: Timer?

    private var locationManager: CLLocationManager!
    private var fakeUserLocations: [CLLocation]?
    private var currentFakeUserLocationsIndex = 0
    private var locationUpdateDelayInSecs = 0
    
    deinit {
        if (locationManager != nil) {
            locationManager?.delegate = nil
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        title = NSLocalizedString("StatisticsViewController.title", comment: "")
        enableLocationLabel.text = NSLocalizedString("StatisticsViewController.location", comment: "")
        simulateLocationButton.setTitle(NSLocalizedString("StatisticsViewController.simulateLocation", comment: ""), for: .normal)
        placeAddLabel.text = NSLocalizedString("ContentMapViewController.placeAddLabel", comment: "")
        initFakeLocations()
        locationUpdateDelayInSecs = 2

        mapView?.loadMap()
        mapView?.setStatisticsLogCamera(true)
        mapView?.setStatisticsLogLocation(true)
        mapView?.setStatisticsLogInterest(true)
        mapView?.setStatisticsLog(true)
        
        // IMPORTANT: Ensure the mapView starts below the navigation bar, otherwise
        // VisioMove Essential's search view button will be hidden.
        if responds(to: #selector(getter: UIViewController.edgesForExtendedLayout)) {
            edgesForExtendedLayout = []
        }
    }
    
    func initFakeLocations() {
        fakeUserLocations = [CLLocation]()
        fakeUserLocations?.append(getLocationLat(45.7414425, lon: 4.8817110, alt: 0, bearing: 0, accuracy: 25)!)
        fakeUserLocations?.append(getLocationLat(45.7415096, lon: 4.8813725, alt: 0, bearing: 0, accuracy: 20)!)
        fakeUserLocations?.append(getLocationLat(45.741124, lon: 4.8812375, alt: 0, bearing: 0, accuracy: 15)!)
        fakeUserLocations?.append(getLocationLat(45.7408424, lon: 4.8811657, alt: 0, bearing: 0, accuracy: 10)!)
        fakeUserLocations?.append(getLocationLat(45.7409480, lon: 4.8805126, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7406672, lon: 4.8804190, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405301, lon: 4.8804921, alt: 0, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405301, lon: 4.8804921, alt: 4, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7405229, lon: 4.8806677, alt: 4, bearing: 0, accuracy: 5)!)
        fakeUserLocations?.append(getLocationLat(45.7407234, lon: 4.8810041, alt: 4, bearing: 0, accuracy: 5)!)
    }
    
    func getLocationLat(_ lat: Double, lon: Double, alt: Double, bearing: Double, accuracy accuarcy: Double) -> CLLocation? {
        let lCLLocation = CLLocation(
            coordinate: CLLocationCoordinate2DMake(CLLocationDegrees(lat), CLLocationDegrees(lon)),
            altitude: CLLocationDistance(alt /* will determine which floor is associated with the location */),
            horizontalAccuracy: CLLocationAccuracy(accuarcy /* will control the size of the accuracy disc */),
            verticalAccuracy: CLLocationAccuracy(0 /* when negative, will force the location to be outside */),
            course: CLLocationDirection(bearing /* will update the camera heading */),
            speed: CLLocationSpeed(5), timestamp: Date())
        return lCLLocation
    }
    
    @IBAction func startLocationSimulation(_ sender: Any) {
        currentFakeUserLocationsIndex = 0
        Timer.scheduledTimer(timeInterval: 0, target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
    }
    
    @objc func simulateLocation(_ pTimer: Timer?) {
        if currentFakeUserLocationsIndex < fakeUserLocations!.count {
            let lLoc : CLLocation = (fakeUserLocations?[currentFakeUserLocationsIndex])!
            let lLocations : [CLLocation] = [lLoc]
            if (locationManager == nil) {
                locationManager = CLLocationManager()
            }
            locationManager(locationManager, didUpdateLocations: lLocations)
            currentFakeUserLocationsIndex += 1
            Timer.scheduledTimer(timeInterval: TimeInterval(locationUpdateDelayInSecs), target: self, selector: #selector(simulateLocation(_:)), userInfo: nil, repeats: false)
        }
    }
    
    @IBAction func enableLocation(_ sender: Any) {

        if !(locationManager != nil) {
            locationManager = CLLocationManager()
            locationManager.delegate = self
            locationManager?.desiredAccuracy = kCLLocationAccuracyBestForNavigation
            // This is required in iOS8 in order to prompt the user to accept
            // to use the localisation
            if locationManager.responds(to: #selector(CLLocationManager.requestWhenInUseAuthorization)) {
                locationManager?.requestWhenInUseAuthorization()
            }
        }

        if (sender as AnyObject).isOn {
            locationManager?.startUpdatingLocation()
        } else {
            locationManager?.stopUpdatingLocation()
            mapView?.update(nil)
        }
    }
    
    
    @IBAction func placeAddSwitchAction(_ sender: Any) {
        let lSwitch = sender as? UISwitch
        let lCatOnline = "lCatOnline"

        if lSwitch?.isOn == true {
            var lPositions: [AnyHashable] = []

            // lCatOnline
            do {

                let lPos = VMEPosition(latitude: 45.74140, longitude: 4.88349, altitude: 0.0, scene: VMESceneContext.init())
                lPositions.append(lPos)

                let lIconUrl = URL(string: "https://i.pinimg.com/originals/5c/b9/18/5cb918952f184853961ea83597082f38.png")

                let placeData = [
                    "name" : "Cat - Online",
                    "description" : "https://en.wikipedia.org/wiki/Cat",
                    "icon" : lIconUrl?.absoluteString as Any,
                    "categories" : ["2", "3", "99"]
                    ]
                
                mapView?.addPlaceID(lCatOnline,
                    imageURL: lIconUrl,
                    data: placeData,
                    position: lPos!,
                    size: VMEPlaceSize(scale: 1.0),
                    anchorMode: VMEPlaceAnchorMode.bottomCenter,
                    altitudeMode: VMEPlaceAltitudeMode.relative,
                    displayMode: VMEPlaceDisplayMode.overlay,
                    orientation: VMEPlaceOrientation.placeOrientationFacing(),
                    visibilityRamp: VMEPlaceVisibilityRamp())
                
                mapView?.setPlaceID(lCatOnline, size: VMEPlaceSize(scale: 20.0), animated: true)

                if !(mCatMoverTimer != nil) {
                    mapView?.setStatisticsTrackedPlaceIDs([lCatOnline])
                    mCatMoverTimer = Timer.scheduledTimer(timeInterval: 2, target: self, selector: #selector(moveCat), userInfo: nil, repeats: true)
                }
            }
            
            let lPadding: CGFloat = 50
            let lCameraUpdate = VMECameraUpdate(builderBlock: { builder in
                    builder?.targets = lPositions
                    builder?.paddingTop = lPadding
                    builder?.paddingBottom = lPadding
                    builder?.paddingLeft = lPadding
                    builder?.paddingRight = lPadding
            })!
            mapView?.animateCamera(lCameraUpdate)
        } else {
            mapView?.removePlaceID(lCatOnline)
            mapView?.setStatisticsTrackedPlaceIDs([])
        }
    }
    
    @objc func moveCat() {
        let lCatOnline = "lCatOnline"

        catMoverIndex += 1
        if catMoverIndex > 5 {
            catMoverIndex = 0
        }
        let lat : Double = (45.74140 + Double(kLatMoverArr[catMoverIndex]) + (Double(arc4random_uniform(1000)) - 500.0) / 1000000.0)
        let lon : Double = (4.88349 + Double(kLonMoverArr[catMoverIndex]) + (Double(arc4random_uniform(1000)) - 500.0) / 1000000.0)
        let lPos = VMEPosition(latitude: lat, longitude: lon, altitude: 0.0, scene: VMESceneContext.init())!

        mapView?.setPlaceID(lCatOnline, position: lPos, animated: true)
    }
    
    // MARK: - CLLocationManagerDelegate
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        let lCLLocation = locations.last!
        let lVMELocation : VMELocation? = (mapView?.createLocation(from: lCLLocation))
        mapView?.update(lVMELocation)
    }
    
}

